package engine;

import commons.*;
import parsing.IParser;
import parsing.ParserFactory;
import analysis.IAnalyser;

import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.List;

import analysis.AnalyserFactory;
import visual.IVisualizer;
import visual.VisualizerFactory;

/**
* <h1>ValuePair</h1>
* Helper Class which in fact is data
* structure that holds all the information
* for a point in a time series
*
* @version 1.0
* @since   2017-07-22
*/
public final class MainEngine implements IEngine{
	private IParser parser;
	private IAnalyser analyser;
	private IVisualizer visualizer;

	//TODO: think of the extension, e.g., single timeseries -> many timeSeries?
	private TimeLine workingTimeLine;
	
	public MainEngine(){
		AnalyserFactory analyserF = new AnalyserFactory();
		ParserFactory parserF = new ParserFactory();
		
		parser = parserF.createParser("SimpleTextParser");
		analyser = analyserF.createAnalyser("NaiveAnalyser");
 
		workingTimeLine = new TimeLine();

	//cannot create visualizer here, because timeLine is null for the moment;
		//moreover, we want to have > 1 of visualizations
	}

	public IParser getParser(){return parser;}
	public IAnalyser getAnalyzer(){return analyser;}
	public IVisualizer getVisualizer(){return visualizer;}
	
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public int setTimeLine(String fileName){
		try {
			workingTimeLine = parser.parse(fileName);
		} catch (FileNotFoundException e) {
			System.out.println("Problem opening file " + fileName + "\n");
			return -1;
			//System.exit(0);
		}
		if (workingTimeLine == null) {
			System.out.println("Sth went wrong in setting the TimeLine");
			return -1;
		}
		return workingTimeLine.getValues().size();
	}//end setTimeLine
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public int producePhases() {
		int numPhases = 0;
		numPhases = analyser.producePhasesFromTimeLine(workingTimeLine).size();
		return numPhases;
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<String> reportPhases(){
		return analyser.reportPhaseTextually();
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public String consoleVerticalReport(){
		return workingTimeLine.consoleVerticalReport();
	}
	
	/**
	 * {@inheritDoc} 
	 */
	@Override
	public void setVisualizer(String className){
		VisualizerFactory visualF = new VisualizerFactory();
		visualizer = visualF.createVisualizer(className, workingTimeLine); 
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public char[][] visualize(PrintStream printstream){
		return visualizer.visualize(printstream);
	}
	
	//public TimeLine getTimeLine(){return workingTimeLine;}

}//end MainEngine
